use anyhow::Result;

use khors_config::ConfigModule;
use khors_app::app::App;
use khors_graphics::renderer::RenderModule;
use khors_window::WindowModule;
use tokio::runtime::Builder;
use winit::event_loop::{ControlFlow, EventLoopBuilder};

fn main() -> Result<()> {
    let event_loop = EventLoopBuilder::new().build()?;

    let runtime = Builder::new_multi_thread().enable_all().build()?;
    // let (event_tx, event_rx) = flume::unbounded();

    runtime.block_on(async {
        runtime.spawn(async move {
            loop {
                std::thread::sleep(std::time::Duration::from_secs(1));
                // let _event = event_rx.recv_async().await.unwrap();
                // println!(
                //     "Tokio got event: {:?} on thread: {:?}",
                //     event,
                //     std::thread::current().id()
                // );
            }
        });
    });

    let mut app = App::new(); // TODO: Move renderer into App
    app.create_window(&event_loop);

    app.push_module(ConfigModule::new);
    app.push_module(WindowModule::new);
    app.push_render_module(RenderModule::new);

    event_loop.run(move |event, elwt| {
        elwt.set_control_flow(ControlFlow::Poll);

        if app
            .process_event_loop(event, elwt)
            .expect("Execution failed")
        {
            elwt.exit();
        }

        // event_tx.send(event.clone()).unwrap();
    })?;

    Ok(())
}
