use crate::debug_gui::DebugGuiStack;
use anyhow::Result;
use flax::World;
use khors_core::events::Events;
use std::time::Duration;

// THREAD LOCAL STUFF

pub trait RenderModule {
    fn on_update(
        &mut self,
        gui_stack: &mut DebugGuiStack,
        vk_context: &mut vulkano_util::context::VulkanoContext,
        vk_windows: &mut vulkano_util::window::VulkanoWindows,
        world: &mut World,
        events: &mut Events,
        frame_time: Duration,
    ) -> Result<()>;
}

pub struct RenderModulesStack {
    modules: Vec<Box<dyn RenderModule>>,
}

impl RenderModulesStack {
    pub fn new() -> Self {
        Self {
            modules: Vec::new(),
        }
    }

    pub fn iter(&self) -> std::slice::Iter<Box<dyn RenderModule>> {
        self.modules.iter()
    }

    pub fn iter_mut(&mut self) -> std::slice::IterMut<Box<dyn RenderModule>> {
        self.modules.iter_mut()
    }

    pub fn push<T: 'static + RenderModule>(&mut self, layer: T) {
        let layer = Box::new(layer);
        self.modules.push(layer);
    }

    pub fn insert<T: 'static + RenderModule>(&mut self, index: usize, layer: T) {
        let layer = Box::new(layer);
        self.modules.insert(index, layer);
    }
}

impl Default for RenderModulesStack {
    fn default() -> Self {
        Self::new()
    }
}

impl<'a> IntoIterator for &'a RenderModulesStack {
    type Item = &'a Box<dyn RenderModule>;

    type IntoIter = std::slice::Iter<'a, Box<dyn RenderModule>>;

    fn into_iter(self) -> Self::IntoIter {
        self.iter()
    }
}

impl<'a> IntoIterator for &'a mut RenderModulesStack {
    type Item = &'a mut Box<dyn RenderModule>;

    type IntoIter = std::slice::IterMut<'a, Box<dyn RenderModule>>;

    fn into_iter(self) -> Self::IntoIter {
        self.iter_mut()
    }
}
